@@ -42,6 +42,8 @@
 #include "Logger.h"
 #include "Misc.h"
 #include "OverrideCursor.h"
+#include "PersistentMemory.h"
+#include "Settings.h"
 #ifndef gmic_core
 #include "CImg.h"
 #endif
@@ -56,7 +58,7 @@ GmicProcessor::GmicProcessor(QObject * parent) : QObject(parent)
   _gmicImages = new cimg_library::CImgList<gmic_pixel_type>;
   _previewImage = new cimg_library::CImg<float>;
   _waitingCursorTimer.setSingleShot(true);
-  connect(&_waitingCursorTimer, SIGNAL(timeout()), this, SLOT(showWaitingCursor()));
+  connect(&_waitingCursorTimer, &QTimer::timeout, this, &GmicProcessor::showWaitingCursor);
   cimg_library::cimg::srand();
   _previewRandomSeed = cimg_library::cimg::_rand();
   _lastAppliedCommandInOutState = InputOutputState::Unspecified;
@@ -82,8 +84,13 @@ void GmicProcessor::execute()
   _gmicImages->assign();
   if ((_filterContext.requestType == FilterContext::RequestType::Preview) || //
       (_filterContext.requestType == FilterContext::RequestType::SynchronousPreview)) {
-    CroppedImageListProxy::get(*_gmicImages, imageNames, rect.x, rect.y, rect.w, rect.h, _filterContext.inputOutputState.inputMode, _filterContext.zoomFactor);
-    updateImageNames(imageNames);
+    if (_filterContext.previewFromFullImage) {
+      CroppedImageListProxy::get(*_gmicImages, imageNames, 0.0, 0.0, 1.0, 1.0, _filterContext.inputOutputState.inputMode, 1.0);
+      updateImageNames(imageNames);
+    } else {
+      CroppedImageListProxy::get(*_gmicImages, imageNames, rect.x, rect.y, rect.w, rect.h, _filterContext.inputOutputState.inputMode, _filterContext.zoomFactor);
+      updateImageNames(imageNames);
+    }
   } else {
     CroppedImageListProxy::get(*_gmicImages, imageNames, rect.x, rect.y, rect.w, rect.h, _filterContext.inputOutputState.inputMode, 1.0);
   }
@@ -91,15 +98,50 @@ void GmicProcessor::execute()
   const InputOutputState & io = _filterContext.inputOutputState;
   QString env = QString("_input_layers=%1").arg(static_cast<int>(io.inputMode));
   env += QString(" _output_mode=%1").arg(static_cast<int>(io.outputMode));
-  env += QString(" _output_messages=%1").arg(static_cast<int>(_filterContext.outputMessageMode));
+  env += QString(" _output_messages=%1").arg(static_cast<int>(Settings::outputMessageMode()));
   if ((_filterContext.requestType == FilterContext::RequestType::Preview) || //
       (_filterContext.requestType == FilterContext::RequestType::SynchronousPreview)) {
-    env += QString(" _preview_width=%1").arg(_filterContext.previewWidth);
-    env += QString(" _preview_height=%1").arg(_filterContext.previewHeight);
+    env += QString(" _preview_area_width=%1").arg(_filterContext.previewWindowWidth);
+    env += QString(" _preview_area_height=%1").arg(_filterContext.previewWindowHeight);
     env += QString(" _preview_timeout=%1").arg(_filterContext.previewTimeout);
   }
+  int maxWidth;
+  int maxHeight;
+  int preview_x0;
+  int preview_y0;
+  int preview_x1;
+  int preview_y1;
+  QSize previewSize;
+  LayersExtentProxy::getExtent(_filterContext.inputOutputState.inputMode, maxWidth, maxHeight);
+  if (_filterContext.previewFromFullImage) {
+    preview_x0 = static_cast<int>(rect.x * maxWidth);
+    preview_y0 = static_cast<int>(rect.y * maxHeight);
+    preview_x1 = preview_x0 + std::min(maxWidth, static_cast<int>(1 + std::ceil(maxWidth * rect.w))) - 1;
+    preview_y1 = preview_y0 + std::min(maxHeight, static_cast<int>(1 + std::ceil(maxHeight * rect.h))) - 1;
+    previewSize = QSize(1 + preview_x1 - preview_x0, 1 + preview_y1 - preview_y0);
+    if (_filterContext.zoomFactor < 1.0) {
+      previewSize = QSize(static_cast<int>(std::round(previewSize.width() * _filterContext.zoomFactor)), //
+                          static_cast<int>(std::round(previewSize.height() * _filterContext.zoomFactor)));
+    }
+  } else {
+    if (_filterContext.zoomFactor < 1.0) {
+      maxWidth = static_cast<int>(std::round(maxWidth * _filterContext.zoomFactor));
+      maxHeight = static_cast<int>(std::round(maxHeight * _filterContext.zoomFactor));
+    }
+    preview_x0 = 0;
+    preview_y0 = 0;
+    preview_x1 = std::min(maxWidth, static_cast<int>(1 + std::ceil(maxWidth * rect.w))) - 1;
+    preview_y1 = std::min(maxHeight, static_cast<int>(1 + std::ceil(maxHeight * rect.h))) - 1;
+    previewSize = QSize(1 + preview_x1 - preview_x0, 1 + preview_y1 - preview_y0);
+  }
+  env += QString(" _preview_x0=%1").arg(preview_x0);
+  env += QString(" _preview_y0=%1").arg(preview_y0);
+  env += QString(" _preview_x1=%1").arg(preview_x1);
+  env += QString(" _preview_y1=%1").arg(preview_y1);
+  env += QString(" _preview_width=%1").arg(previewSize.width());
+  env += QString(" _preview_height=%1").arg(previewSize.height());
   if (_filterContext.requestType == FilterContext::RequestType::SynchronousPreview) {
-    FilterSyncRunner runner(this, _filterContext.filterCommand, _filterContext.filterArguments, env, _filterContext.outputMessageMode);
+    FilterSyncRunner runner(this, _filterContext.filterCommand, _filterContext.filterArguments, env);
     runner.swapImages(*_gmicImages);
     runner.setImageNames(imageNames);
     runner.setLogSuffix("preview");
@@ -110,11 +152,11 @@ void GmicProcessor::execute()
     manageSynchonousRunner(runner);
     recordPreviewFilterExecutionDurationMS((int)_filterExecutionTime.elapsed());
   } else if (_filterContext.requestType == FilterContext::RequestType::Preview) {
-    _filterThread = new FilterThread(this, _filterContext.filterCommand, _filterContext.filterArguments, env, _filterContext.outputMessageMode);
+    _filterThread = new FilterThread(this, _filterContext.filterCommand, _filterContext.filterArguments, env);
     _filterThread->swapImages(*_gmicImages);
     _filterThread->setImageNames(imageNames);
     _filterThread->setLogSuffix("preview");
-    connect(_filterThread, SIGNAL(finished()), this, SLOT(onPreviewThreadFinished()), Qt::QueuedConnection);
+    connect(_filterThread, &FilterThread::finished, this, &GmicProcessor::onPreviewThreadFinished, Qt::QueuedConnection);
     cimg_library::cimg::srand();
     _previewRandomSeed = cimg_library::cimg::_rand();
     _filterExecutionTime.restart();
@@ -125,11 +167,11 @@ void GmicProcessor::execute()
     _lastAppliedCommand = _filterContext.filterCommand;
     _lastAppliedCommandArguments = _filterContext.filterArguments;
     _lastAppliedCommandInOutState = _filterContext.inputOutputState;
-    _filterThread = new FilterThread(this, _filterContext.filterCommand, _filterContext.filterArguments, env, _filterContext.outputMessageMode);
+    _filterThread = new FilterThread(this, _filterContext.filterCommand, _filterContext.filterArguments, env);
     _filterThread->swapImages(*_gmicImages);
     _filterThread->setImageNames(imageNames);
     _filterThread->setLogSuffix("apply");
-    connect(_filterThread, SIGNAL(finished()), this, SLOT(onApplyThreadFinished()), Qt::QueuedConnection);
+    connect(_filterThread, &FilterThread::finished, this, &GmicProcessor::onApplyThreadFinished, Qt::QueuedConnection);
     cimg_library::cimg::srand(_previewRandomSeed);
     _filterThread->start();
   }
@@ -284,6 +326,7 @@ void GmicProcessor::onPreviewThreadFinished()
   _parametersVisibilityStates = _filterThread->parametersVisibilityStates();
   _gmicImages->assign();
   _filterThread->swapImages(*_gmicImages);
+  PersistentMemory::move_from(_filterThread->persistentMemoryOutput());
   unsigned int badSpectrumIndex = 0;
   bool correctSpectrums = checkImageSpectrumAtMost4(*_gmicImages, badSpectrumIndex);
   if (correctSpectrums) {
@@ -325,6 +368,7 @@ void GmicProcessor::onApplyThreadFinished()
     emit fullImageProcessingFailed(message);
   } else {
     _filterThread->swapImages(*_gmicImages);
+    PersistentMemory::move_from(_filterThread->persistentMemoryOutput());
     unsigned int badSpectrumIndex = 0;
     bool correctSpectrums = checkImageSpectrumAtMost4(*_gmicImages, badSpectrumIndex);
     if (!correctSpectrums) {
@@ -406,7 +450,7 @@ void GmicProcessor::abortCurrentFilterThread()
     return;
   }
   _filterThread->disconnect(this);
-  connect(_filterThread, SIGNAL(finished()), this, SLOT(onAbortedThreadFinished()));
+  connect(_filterThread, &FilterThread::finished, this, &GmicProcessor::onAbortedThreadFinished);
   _unfinishedAbortedThreads.push_back(_filterThread);
   _filterThread->abortGmic();
   _filterThread = nullptr;
@@ -428,6 +472,7 @@ void GmicProcessor::manageSynchonousRunner(FilterSyncRunner & runner)
   _parametersVisibilityStates = runner.parametersVisibilityStates();
   _gmicImages->assign();
   runner.swapImages(*_gmicImages);
+  PersistentMemory::move_from(runner.persistentMemoryOutput());
   for (unsigned int i = 0; i < _gmicImages->size(); ++i) {
     GmicQtHost::applyColorProfile((*_gmicImages)[i]);
   }
